{% extends 'infra_optim/base.html' %}
{% load i18n %}
{% block title %}{% trans "Skip Action" %}{% endblock %}
{% block page_header %}
  {% include "horizon/common/_page_header.html" with title=_("Skip Action") %}
{% endblock page_header %}
{% block main %}
  {% include "infra_optim/actions/_skip.html" %}
{% endblock %}
